package itso.cics.eci.j2ee.testercci;

import com.ibm.connector2.cics.*;
import com.ibm.ctg.client.T;

import itso.cics.eci.j2ee.testercci.JavaStringRecord;

import javax.resource.cci.*;
import javax.resource.*;
import javax.ejb.CreateException;

/**
 * Bean implementation class for Enterprise Bean: CTGTesterCCI
 */
public class CTGTesterCCIBean implements javax.ejb.SessionBean {
	private javax.ejb.SessionContext mySessionCtx;
	private ConnectionFactory cf = null;
	
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
		javax.naming.Context ic=null;
		
		try {
			ic = new javax.naming.InitialContext();
		} catch (Exception e) {
			e.printStackTrace();
			throw new CreateException("ejbCreate() Exception occurred obtaining InitialContext. Exception Message=" + e.getMessage());
		}
		
		// Create an ECI ConnectionFactory instance.
		// At runtime, the Application Server must find a JNDI definition for 
		// the specified resource reference. 
		Connection eciConn = null;
		// Define ConnectionFactory resource reference		
		String cfRef = new String("java:comp/env/ECI");
		
		try {
			cf = (ConnectionFactory) ic.lookup(cfRef);
		} catch (Exception e) {
			e.printStackTrace();
			throw new CreateException("Lookup for " + cfRef
					+ " failed. Exception Message=" + e.getMessage());
		}
		if (cf == null) {
			throw new CreateException("Lookup for " + cfRef
					+ " resulted in a null reference");
		}
	}
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}

	/**
	 * Get a connection to the EIS using a ConnectionFactory.
	 */
	private Connection getConnection(EciRequestDetails eRD, ConnectionFactory targetCF)
			throws Exception {
		// we need to get the Connection Factory and from there a
		// connection and interaction.  
		Connection eciConn = null;

		try {
			ECIConnectionSpec ecf = new ECIConnectionSpec();

			// set security details if specified
			if (eRD.username.equals("") == false) {
				ecf.setUserName(eRD.username);
			}
			if (eRD.password.equals("") == false) {
				ecf.setPassword(eRD.password);
			}
			
			// get a connection from the connection factory with
			// the above security attributes
			eciConn = (Connection) targetCF.getConnection(ecf);
		} catch (Exception e) {
			e.printStackTrace();
			throw new Exception(
					"failed to get connection from ECI Factory. Exception Message="
					+ e.getMessage() + " ConnectionFactory="+targetCF.toString());
		}
		return eciConn;
	}

	private Interaction getInteraction(Connection eciConn) throws Exception {
		Interaction eciInt = null;
		try {
			eciInt = (Interaction) eciConn.createInteraction();
		} catch (Exception e) {
			e.printStackTrace();
			throw new Exception(
					"failed to get interaction from ECI Connection. Exception Message="
							+ e.getMessage());
		}
		return eciInt;
	}

	/**
	 * drops the connection to the EIS
	 */
	private void dropConnection(Connection eciConn, Interaction eciInt) {
		// tidy up the interaction and connection and set our references to
		// null.
		try {
			eciInt.close();
			eciConn.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * This is the main method that calls flowEciRequest and returns the
	 * resulting commarea data to the caller.
	 * 
	 * @param eciRD
	 *            encapsulates the JSP ECI request details
	 */
	public String execute(EciRequestDetails eciRD) throws ResourceException,
			Exception {
		JavaStringRecord jsrCA = null;

		// alter application trace
		T.setTimingOn(true);
		/*
		 * The managed resource adapter will not output any tracing unless T
		 * debug is enabled. Therefore enabling debug will not only debug
		 * Application side methods, but the CCI trace at the level set in the
		 * managed environment.
		 */

		if (eciRD.appTraceBool) {
			T.setDebugOn(true);
		} else {
			T.setDebugOn(false);
		}

		// Send ECI request to a named Connection Factory reference
		jsrCA = flowEciRequest(eciRD, cf);

		// return the commarea from the last executed program (iteration)
		if (jsrCA != null) {
			return jsrCA.getText();
		} else {
			return "";
		}
	}

	private JavaStringRecord flowEciRequest(EciRequestDetails eciRD, ConnectionFactory targetCF) throws Exception {
       	// create an Interaction spec to work with, This is specific to ECI
		ECIInteractionSpec eSpec = new ECIInteractionSpec();
		Connection eciConn = null;
		Interaction eciInt = null;
		JavaStringRecord jsr = null;

		// get managed connection
		eciConn = getConnection(eciRD, targetCF);
		eciInt = getInteraction(eciConn);

		// setup the interactionSpec.
		eSpec.setFunctionName(eciRD.funcName);
		eSpec.setCommareaLength(eciRD.commareaLengthInt);
		eSpec.setInteractionVerb(ECIInteractionSpec.SYNC_SEND_RECEIVE);

		// It may be desirable to set the reply length to the
		// same size as the commarea length - if it is expected
		// that the full comarea will be utilized; however, doing 
		// so disables the commarea null-stripping optimization.
		// eSpec.setReplyLength(eciRD.commareaLengthInt);

		// optionally override the mirror transid
		if (eciRD.mirror.equals("") == false) {
			eSpec.setTPNName(eciRD.mirror);
		}

		// ensure sensible iterations value 
		if (eciRD.iterationsInt < 1) {
			eciRD.iterationsInt = 1;
		}

		// create a record for use
		jsr = new JavaStringRecord(eciRD.encoding);

		// execute the program a number of times
		for (int i = 0; i < eciRD.iterationsInt; i++) {
			// set input data if we have any
			if (eciRD.commareaLengthInt > 0) {
				jsr.setText(eciRD.commarea);
			}

			// make the call
			try {
				eciInt.execute(eSpec, jsr, jsr);
			} catch (ResourceException re) {
				// output the stacktrace and re-throw it back to the client
				re.printStackTrace();
				// depending upon the kind of exception thrown, we may decide 
				// to abandon the current transaction by calling setRollbckOnly()
				// on the current context. If this request is not part of a local
				// or global transaction, there will be a second exception to 
				// catch. If so, we will log the exception details, but otherwise 
				// absorb it.
				
				// in the case of a CICSTxnAbendException, we will write a log 
				// message and rollback any current transaction.
				if (re instanceof com.ibm.connector2.cics.CICSTxnAbendException )
				{
					System.err.println("CTGTesterCCIBean.flowEciRequest: CICS ABEND detected."+
							" Connection Factory="+targetCF.toString());
					try {
						mySessionCtx.setRollbackOnly();
					} catch(IllegalStateException ise) {					
						ise.printStackTrace();
					}
				} else {
					// Write generic failure log message
					System.err.println("CTGTesterCCIBean.flowEciRequest: ResourceException detected."+
							" Connection Factory="+targetCF.toString());
				}
				
				// clean up interaction and connection references
				dropConnection(eciConn, eciInt);
				eciInt = null;
				eciConn = null;
				// throw exception back to caller
				ResourceException re2 = new ResourceException(re.getMessage()+
						" ConnectionFactory="+targetCF.toString());
				re2.initCause(re);
				throw re2;
			}
		}
		dropConnection(eciConn, eciInt);
		eciInt = null;
		eciConn = null;
		return jsr;
	}
}
