package itso.cics.eci.j2ee.testercci;

import javax.servlet.http.*;
import javax.servlet.*;
import javax.naming.*;
import javax.rmi.*;

import java.util.Date;
import java.io.UnsupportedEncodingException;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

/**
 * Servlet which calls CTGTesterCCI session bean to invoke an ECI program on CICS.
 * Implementation class for Servlet: CTGTesterCCIServlet
 * 
 * Was enhanced by Hanno Eichelberger to handle numbers as input for the
 * COMMAREA which can be handled by the CTG218 prime number testing program 
 * 
 * @web.servlet name="PrimeCTGServlet"
 *              display-name="PrimeCTGServlet"
 * 
 * @web.servlet-mapping url-pattern="/PrimeCTGServlet"
 *  
 */

public class PrimeCTGServlet extends HttpServlet {
	// date loaded into application server
	private Date loadedDate;

	// JNDI name of EJB
	private String ejbJNDIName;

	// jsp names
	private static final String jspResults = "results.jsp";

	private static final String jspError = "error.jsp";

	// output messages buffer
	private String messages = "";

	// names of attributes set in the request
	private static final String attrResultsString = "results";

	private static final String attrDefaultResultsString = "defaultResults";

	private static final String attrHexResultsString = "hexResults";

	private static final String attrErrorException = "errorException";

	private static final String attrErrorMessages = "errorMessages";

	private static final String attrLastLoaded = "lastLoaded";

	private static final String attrCicsProgram = "funcName";

	private static final String attrGatewayURL = "gatewayURL";

	private static final String attrGatewayPort = "gatewayPort";

	private static final String attrServer = "server";

	private static final String attrUsername = "username";

	private static final String attrMirror = "mirror";

	private static final String attrCommareaLength = "commareaLength";

	private static final String attrCommarea = "commareaInput";

	private static final String attrEncoding = "encoding";

	private static final String attrTrace = "trace";

	private static final String attrIterations = "iterations";

	private static final String attrAppTrace = "appTrace";

	private static final String attrRequestTime = "requestTime";

	/**
	 * Initialise the servlet.
	 */
	public void init(ServletConfig sc) throws ServletException {
		// call HttpServlet init method
		super.init(sc);

		// Set time of loading
		loadedDate = new Date();
	}

	/**
	 * Do a GET request. Hand over to common processing routine.
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		try {
			processRequest(request, response);
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	/**
	 * Do a POST request. Hand over to common processing routine.
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		try {
			processRequest(request, response);
		} catch (Throwable e) {
			e.printStackTrace();
		}

	}

	/**
	 * Common processing routine.
	 */
	public void processRequest(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {
		// date loaded into application server
		Date beforeReqTime, afterReqTime;

		EciRequestDetails eciReq1 = new EciRequestDetails();

		// output messages buffer
		messages = "";

		// the jsp to forward to for displaying the results
		String jsp = jspResults;

		// values for strings to be converted into
		int commareaLengthInt = 0;
		int gatewayDaemonPort = 2006;
		int traceInt = 0;
		int iterationsInt = 1;
		boolean appTraceBool = false;

		//---- Hanno Eichelberger
		// Check if the input is a number and transforms
		// it in the number of lenght 7.
		// E.g. transform input 17 to 0000017.
		String number = request.getParameter("commareaInput")+"";
		int intNumber = 1;
		try{
			intNumber = Integer.parseInt(number.replaceAll("^0*",""));
		}catch(Exception e){
			
		}
		number = ""+intNumber;
		for (int i=0;i<7-(intNumber+"").length();i++)
			number = "0"+number;
		
		// retrieve HTTP request parameters
		MapRequest2Details(request, eciReq1, "");

		// Set the changed number input (look above) as CTG Bean request input
		eciReq1.commarea=number;
		eciReq1.commareaLengthInt=number.length();		
		//------------
		
		// set last loaded date in the request
		request.setAttribute(attrLastLoaded, loadedDate.toString());

		// set error messages in the request
		request.setAttribute(attrErrorMessages, messages);

		// execute the CICS request using the session bean
		try {
			// lookup our session bean
			Context ic = new InitialContext();
			Object or = ic.lookup("java:comp/env/ejb/CTGTesterCCI");
			CTGTesterCCI tester = null;
			if (or != null) {
				CTGTesterCCIHome home = (CTGTesterCCIHome) PortableRemoteObject
						.narrow(or, CTGTesterCCIHome.class);
				tester = home.create();
			}
			beforeReqTime = new Date();
			String results = tester.execute(eciReq1);
			afterReqTime = new Date();
			long reqTimeLong = afterReqTime.getTime() - beforeReqTime.getTime();

			// set result string attribute
			request.setAttribute(attrResultsString, results);
			request.setAttribute(attrRequestTime, Long.toString(reqTimeLong));

			// set result string in default encoding attribute
			try {
				request.setAttribute(attrDefaultResultsString, new String(
						results.getBytes(eciReq1.encoding)));
			} catch (UnsupportedEncodingException ex) {

			}

			// set result string in hex encoding attribute
			byte[] data;
			try {
				// get original data by doing a get bytes on the string using
				// the encoding
				// supplied, which was used to convert the data to a string in
				// the JavaStringRecord class.
				data = results.getBytes(eciReq1.encoding);

				// set request attribute for hex data
				request
						.setAttribute(attrHexResultsString,
								byteArrayToHex(data));
			} catch (UnsupportedEncodingException ex) {

			}

		} catch (Throwable t) {
			// exception occured, set exception message in request with
			// timestamp
			String nowStr = "";
			try {
				SimpleDateFormat sdf = (SimpleDateFormat) DateFormat
						.getTimeInstance();
				sdf.applyPattern("MM/dd/yy HH:mm:ss:SSS");
				nowStr = sdf.format(new Date());
			} catch (ClassCastException ex2) {
				// no SimpleDateFormat instance for this locale

				// format with Date toString
				nowStr = (new Date()).toString();
			}
			request.setAttribute(attrErrorException, nowStr + " : " + t);

			// change jsp to use to the error jsp
			jsp = jspError;
		} finally {
			// route to jsp
			ServletContext servletContext = getServletContext();
			RequestDispatcher dispatcher = servletContext
					.getRequestDispatcher(jsp);
			dispatcher.forward(request, response);
		}
	}

	/**
	 * convert a byte array to a string representation of the hex value.
	 */
	private String byteArrayToHex(byte[] data) {
		StringBuffer hex = new StringBuffer();

		for (int i = 0; i < data.length; i++) {
			String hexByte = Integer.toHexString(data[i]);
			if (hexByte.length() == 1) {
				hexByte = "0" + hexByte;
			} else if (hexByte.length() > 2) {
				hexByte = hexByte.substring(hexByte.length() - 2);
			}
			hex.append(hexByte.toUpperCase());
			hex.append(' ');
		}

		return hex.toString();

	}

	private void MapRequest2Details(HttpServletRequest request,
			EciRequestDetails eciReqDet, String suffix) {
		// input parameters
		String iterations = null;
		String appTrace = null;
		String trace = "";
		String commareaLength = null;

		// retrieve HTTP request parameters
		eciReqDet.funcName = request.getParameter("funcName" + suffix);
		eciReqDet.encoding = request.getParameter("encoding" + suffix);
		eciReqDet.commarea = request.getParameter("commareaInput" + suffix);
		commareaLength = request.getParameter("commareaLength" + suffix);

		// convert commarea length to int
		try {
			eciReqDet.commareaLengthInt = Integer.parseInt(commareaLength);

			// set commarea length attribute
			request.setAttribute(attrCommareaLength + suffix, Integer
					.toString(eciReqDet.commareaLengthInt));
		} catch (NumberFormatException ex) {
			messages += "Commarea length not a number<br>";
		}

		iterations = request.getParameter("iterations" + suffix);
		// convert iteration number to int
		if (iterations != null) {
			try {
				eciReqDet.iterationsInt = Integer.parseInt(iterations);
				// set iterations number attribute
				request.setAttribute(attrIterations + suffix, Integer
						.toString(eciReqDet.iterationsInt));
			} catch (NumberFormatException ex) {
				messages += "Iterations not a number<br>";
			}
		}
		appTrace = request.getParameter("appTrace" + suffix);
		if (appTrace != null) {
			eciReqDet.appTraceBool = appTrace.equalsIgnoreCase("on");
		}
		eciReqDet.username = request.getParameter("username" + suffix);
		eciReqDet.password = request.getParameter("password" + suffix);
		eciReqDet.mirror = request.getParameter("mirror" + suffix);

		// set common attributes in the request
		request.setAttribute(attrCicsProgram + suffix, eciReqDet.funcName);
		request.setAttribute(attrEncoding + suffix, eciReqDet.encoding);
		request.setAttribute(attrCommarea + suffix, eciReqDet.commarea);
		request.setAttribute(attrAppTrace + suffix, appTrace);
		request.setAttribute(attrUsername + suffix, eciReqDet.username);
		request.setAttribute(attrMirror + suffix, eciReqDet.mirror);
	}
}
